#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include <qmessagebox.h>

//#include <klocale.h>
//#define _(TEXT) globalKlocale->translate(TEXT)

#include "jabatoBlock.h"

#include <qdir.h>


/* ---------------------------------------------------------------------- */
JabatoStrip::JabatoStrip() {
  blockId = 0;
  dayOfTheWeek = 0;
  beginAtHour = 0;
  beginAtMinute = 0;
  endAtHour = 0;
  endAtMinute = 0;
  recordingStrip = false;
}

bool
JabatoStrip::withinStrip(int dow, int hh, int mm) const {
/*  DEBUG(fprintf(stderr,
        "JabatoStrip::withinStrip(%d,%02d:%02d) --> Strip %d[%02d:%02d]>---<[%02d:%02d] \n",
        dow, hh, mm,
        dayOfTheWeek, beginAtHour,beginAtMinute, endAtHour, endAtMinute)); */

  if(dayOfTheWeek==dow) {
    uint asked = (hh << 6) + mm;
    uint begin = (beginAtHour << 6) + beginAtMinute;
    uint end = (endAtHour << 6) + endAtMinute;
    return ((begin<=asked)&&(asked<end));
  }
  else
    return false;
}

/* ---------------------------------------------------------------------- */

JabatoDirDigger::JabatoDirDigger() {
  DEBUG(fprintf(stderr, "JabatoDirDigger::JabatoDirDigger() \n"));
  nextSound = 0;
  bShuffle = true;
}

JabatoDirDigger::~JabatoDirDigger() {
  DEBUG(fprintf(stderr, "JabatoDirDigger::~JabatoDirDigger() \n"));
}


void
JabatoDirDigger::addDirectory(const char *dirpath, char *filter) {

  QDir d(dirpath,filter);

  PRINTDIRS(fprintf(stdout, "%s\n",dirpath));
  if (d.exists()&&d.isReadable())
  {

     const QFileInfoList *list = d.entryInfoList();
     QFileInfoListIterator it( *list );
     QFileInfo  *filename;

     while ((filename=it.current())) {
       soundList.append(d.absFilePath(filename->fileName()).data());
       ++it;
     }

     d.setNameFilter("*");
     d.setFilter(QDir::Dirs);
     list=d.entryInfoList();
     QFileInfoListIterator dirIt( *list );
     while ((filename=dirIt.current())) {
       if ((strcmp(filename->fileName().data(), ".")!=0) && 
	   (strcmp(filename->fileName().data(), "..")!=0))
         this->addDirectory(d.absFilePath(filename->fileName()).data(), filter);
       ++dirIt;
     }
  }
}


char*
JabatoDirDigger::getSound(){
  DEBUG(fprintf(stderr, "JabatoDirDigger::getSound() "));

  if(soundList.count() > 0) {
    if(nextSound >= accessArray.size())
      this->checkIndexes();
    DEBUG(fprintf(stderr, "%d-->%d\n",nextSound,accessArray[nextSound]));
    return(soundList.at(accessArray[nextSound++]));
  }
  else
    return NULL;
}


char*
JabatoDirDigger::getSoundAt(int i){
  if((-1 < i) && (soundList.count() > (uint)i))
    return(soundList.at(i));
  else
    return NULL;
}


uint
JabatoDirDigger::count() const {
  return soundList.count();
}


void
JabatoDirDigger::init(bool shuffle) {
  DEBUG(fprintf(stderr, "JabatoDirDigger::init()..."));
  bShuffle=shuffle;
  if(!bShuffle)
    nextSound = 0;
  DEBUG(fprintf(stderr, "OK\n"));
}

bool
JabatoDirDigger::isShuffled() {
  return bShuffle;
}

void
JabatoDirDigger::checkIndexes() {
  uint i;

  if(soundList.count()>accessArray.size()) {
    accessArray.resize(soundList.count());
    if (!bShuffle)
      for(i=0;i<accessArray.size();i++) accessArray[i]=i;
  }

  if (bShuffle) {
    srand((unsigned int)time((time_t *)NULL));
    int index;
    for(i=0;i<accessArray.size();i++) {
      while (alreadyInArray((index = rand()%soundList.count()), i));
      DEBUG(fprintf(stderr, "%d->%d ",i,index));
      accessArray[i]=index;
    }
    DEBUG(fprintf(stderr, "%d files, %d indexes\n",soundList.count(),accessArray.size()));
  }

  nextSound = 0;
}


bool
JabatoDirDigger::alreadyInArray(int index, int upTo) {
  int i=0;
  while((accessArray[i] != index) && (i<upTo)) i++;
  return i<upTo;
}


void
JabatoDirDigger::clear() {
  DEBUG(fprintf(stderr, "JabatoDirDigger::clear()..."));
  soundList.clear();
  accessArray.resize(0);
  nextSound = 0;
  bShuffle = true;
}

/* ---------------------------------------------------------------------- */

JabatoBlock::JabatoBlock(int id, char *name) {
  DEBUG(fprintf(stderr, "JabatoBlock::JabatoBlock(%d,%s)\n",id,name));
  m_id=-1;
  songsPerSlot = 1;
  songsPlayed = 1;
  programPlayed = false;
  textTold = false;
  shuffleType = 3;
  m_id = id;
  m_name = name;
  program = "";
  programToLanDiskCmd = "";
  programToWanDiskCmd = "";
  songsBlock=NULL;
  slotsBlock=NULL;
  textsBlock=NULL;
}

JabatoBlock::~JabatoBlock() {
  DEBUG(fprintf(stderr, "JabatoBlock::~JabatoBlock()\n"));
  if(slotsBlock!=NULL)
    delete slotsBlock;
  if(songsBlock!=NULL)
    delete songsBlock;
  if(textsBlock!=NULL)
    delete textsBlock;
}


void
JabatoBlock::setId(int id) {
  m_id = id;
}

void
JabatoBlock::setName(char *name) {
  m_name = name;
}

void
JabatoBlock::setHRef(char *lpstr){
  m_href = lpstr;
}

void
JabatoBlock::setJabatos(char *lpstr){
  m_jabatos = lpstr;
}

void
JabatoBlock::setImage(char *lpstr){
  m_image = lpstr;
}

void
JabatoBlock::setBackImage(char *lpstr){
  m_backimage = lpstr;
}

void
JabatoBlock::setEmail(char *lpstr){
  m_email = lpstr;
}

void
JabatoBlock::setDescription(char *lpstr){
  m_description += lpstr;
}

void
JabatoBlock::setProgramFile(char *filename) {
  program = filename;
}

void
JabatoBlock::setProgramToLanDiskCmd(char *cmd) {
  programToLanDiskCmd = cmd;
}

void
JabatoBlock::setProgramToWanDiskCmd(char *cmd) {
  programToWanDiskCmd = cmd;
}


int
JabatoBlock::getId() const {
  return m_id;
}

const char *
JabatoBlock::getName() const {
  return m_name.data();
}

const char *
JabatoBlock::getHRef() const {
  return m_href.data();
}

const char *
JabatoBlock::getJabatos() const {
  return m_jabatos.data();
}

const char *
JabatoBlock::getImage() const {
  return m_image.data();
}

const char *
JabatoBlock::getBackImage() const {
  return m_backimage.data();
}

const char *
JabatoBlock::getEmail() const {
  return m_email.data();
}

const char *
JabatoBlock::getDescription() const {
  return m_description.data();
}

const char *
JabatoBlock::getProgramFile() const {
  if(program.length()>0)
    return program.data();
  else
    return NULL;
}

void
JabatoBlock::addSongsDirectory(char *dirpath, char *filter) {
  if(songsBlock==NULL)
    songsBlock = new JabatoDirDigger();
  songsBlock->addDirectory(dirpath, filter);
}

void
JabatoBlock::addSlotsDirectory(char *dirpath, char *filter) {
  if(slotsBlock==NULL)
    slotsBlock = new JabatoDirDigger();
  slotsBlock->addDirectory(dirpath, filter);
}

void
JabatoBlock::addTextsDirectory(char *dirpath, char *filter) {
  if(textsBlock==NULL)
    textsBlock = new JabatoDirDigger();
  textsBlock->addDirectory(dirpath, filter);
}

void
JabatoBlock::setSongsPerSlot(int i) {
  songsPerSlot = i;
  songsPlayed = i;
  textTold=false;
}

const int
JabatoBlock::getSongsPerSlot() {
  return songsPerSlot;
}

void
JabatoBlock::setShuffleType(int i) {
  DEBUG(fprintf(stderr, "JabatoBlock::setShuffleType(%d)\n", i));
  if(songsBlock==NULL)
    songsBlock = new JabatoDirDigger();
  if(slotsBlock==NULL)
    slotsBlock = new JabatoDirDigger();
  if(textsBlock==NULL)
    textsBlock = new JabatoDirDigger();
  shuffleType = i;
  if(i==0) {
    songsBlock->init(false);
    slotsBlock->init(false);
    textsBlock->init(false);
  }
  else if(i==1) {
    songsBlock->init(true);
    slotsBlock->init(false);
    textsBlock->init(false);
  }
  else if(i==2) {
    songsBlock->init(false);
    slotsBlock->init(true);
    textsBlock->init(false);
  }
  else if(i==3) {
    songsBlock->init(true);
    slotsBlock->init(true);
    textsBlock->init(false);
  }
  else if(i==4) {
    songsBlock->init(false);
    slotsBlock->init(false);
    textsBlock->init(true);
  }
  else if(i==5) {
    songsBlock->init(true);
    slotsBlock->init(false);
    textsBlock->init(true);
  }
  else if(i==6) {
    songsBlock->init(false);
    slotsBlock->init(true);
    textsBlock->init(true);
  }
  else {
    songsBlock->init(true);
    slotsBlock->init(true);
    textsBlock->init(true);
  }
}

bool
JabatoBlock::songsShuffled() {
  if(songsBlock==NULL)
    return false;
  else
    return songsBlock->isShuffled();
}

bool
JabatoBlock::slotsShuffled() {
  if(slotsBlock==NULL)
    return false;
  else
    return slotsBlock->isShuffled();
}

bool
JabatoBlock::textsShuffled() {
  if(textsBlock==NULL)
    return false;
  else
    return textsBlock->isShuffled();
}

JabatoDirDigger
*JabatoBlock::getSongsBlock(){
  return songsBlock;
}

JabatoDirDigger
*JabatoBlock::getSlotsBlock(){
  return slotsBlock;
}

JabatoDirDigger
*JabatoBlock::getTextsBlock(){
  return textsBlock;
}

void
JabatoBlock::setSongsBlock(JabatoDirDigger *b){
  if(songsBlock!=NULL)
    delete songsBlock;
  songsBlock=b;
}

void
JabatoBlock::setSlotsBlock(JabatoDirDigger *b){
  if(slotsBlock!=NULL)
    delete slotsBlock;
  slotsBlock=b;
}

void
JabatoBlock::setTextsBlock(JabatoDirDigger *b){
  if(textsBlock!=NULL)
    delete textsBlock;
  textsBlock=b;
}

uint
JabatoBlock::songsCount() const {
  if(songsBlock==NULL)
    return 0;
  else
    return songsBlock->count();
}


uint
JabatoBlock::slotsCount() const {
  if(slotsBlock==NULL)
    return 0;
  else
    return slotsBlock->count();
}

uint
JabatoBlock::textsCount() const {
  if(textsBlock==NULL)
    return 0;
  else
    return textsBlock->count();
}

bool
JabatoBlock::isLoaded() const {
  if((slotsBlock==NULL)&&(songsBlock==NULL))
    return false;
  else {
    return ((songsBlock==NULL?0:songsBlock->count())
           +(slotsBlock==NULL?0:slotsBlock->count())
	   +(textsBlock==NULL?0:textsBlock->count()))>0;
  }
}

void
JabatoBlock::updateSound(JabatoSound *snd) {
  uint i;
  fprintf(stderr, "JabatoBlock::updateSound() %s->",m_name.data());
  
    if (!(programPlayed)&&(program.length()>0)) {
      DEBUG(fprintf(stderr, "PROGRAM\n"));
      programPlayed = true;
      snd->setFullPath(program.data());
      snd->setSoundType(JABATOPROGRAM);
      snd->setToLanDiskCmd((char *)programToLanDiskCmd.data());
      snd->setToWanDiskCmd((char *)programToWanDiskCmd.data());
      return;
    }
    
    if(textsBlock!=NULL)
      if(!textTold && 
         (textsBlock->count()>0) && 
	 (songsPlayed>=songsPerSlot)) {
        DEBUG(fprintf(stderr, "TEXT\n"));
        snd->setFullPath(textsBlock->getSound());
      	snd->setSoundType(JABATOTEXT);
      	textTold=true;
	songsPlayed=0;
      	return;
      }
      
    if(slotsBlock!=NULL)
      if(slotsBlock->count()>0)
        if(songsPlayed>=songsPerSlot) {
	  DEBUG(fprintf(stderr, "SLOT\n"));
	  for(i=0; i<slotsBlock->count(); i++) {
  	    snd->setFullPath(slotsBlock->getSound());
	    snd->setSoundType(JABATOSLOT);
  	    if(snd->getTimeInSeconds()>0) {
	      textTold=false;
	      songsPlayed=0;
	      return;
  	    }
  	  }
  	};

    if(songsBlock!= NULL)
      if(songsBlock->count()>0) {
        DEBUG(fprintf(stderr, "SONG\n"));
        snd->setFullPath(songsBlock->getSound());
      	snd->setSoundType(JABATOSONG);
      	songsPlayed++;
      	return;
      }

    if(slotsBlock!= NULL)
      if(slotsBlock->count()>0) {
        DEBUG(fprintf(stderr, "NO SONGS!! SLOT\n"));
        for(i=0; i<slotsBlock->count(); i++) {
          snd->setFullPath(slotsBlock->getSound());
 	  snd->setSoundType(JABATOSLOT);
 	  if(snd->getTimeInSeconds()>0) {
	    songsPlayed=0;
  	    return;
 	  }
 	}
      }

    if (program.length()>0) {
      DEBUG(fprintf(stderr, "NO SONGS OR SLOTS!! PROGRAM BY DEFAULT\n"));
      snd->setFullPath(program.data());
      snd->setSoundType(JABATOPROGRAM);
      return;
    }
    DEBUG(fprintf(stderr, "EMPTY BLOCK!!! (CANNOT UPDATE SOUND)\n"));

}


void
JabatoBlock::init() {
  DEBUG(fprintf(stderr, "JabatoBlock::init()\n"));
  programPlayed = false;
  setShuffleType(shuffleType);
}


void
JabatoBlock::clear() {
  DEBUG(fprintf(stderr, "JabatoBlock::clear()\n"));
  m_id=-1;
  m_name = "";
  songsPerSlot = 1;
  songsPlayed = 1;
  shuffleType = 3;
  programPlayed = false;
  textTold = false;
  program = "";
  programToLanDiskCmd = "";
  programToWanDiskCmd = "";
  songsBlock->clear();
  slotsBlock->clear();
  textsBlock->clear();
}

/* ---------------------------------------------------------------------- */
