#include <stdio.h>
#include <qdatetime.h>

#include "jabatoSound.h"
#include "jabatoFileFeeder.h"


JabatoFileFeeder::JabatoFileFeeder(JabatoSchedule *sch) {
  schedule = sch;
  block = schedule->getDefaultBlock();
  finalSecond = 0;
  m_blockIndex = 0;
  updated = false;
}

JabatoFileFeeder::~JabatoFileFeeder() {
}


bool
JabatoFileFeeder::blockProgram(){
  if(block->getProgramFile()==NULL)
    return false;
  else
    return true;
}

int
JabatoFileFeeder::blockIndex() {
  return m_blockIndex;
}

const char *
JabatoFileFeeder::blockName(){
  return block->getName();
}

const char *
JabatoFileFeeder::blockJabatos(){
  return block->getJabatos();
}

int
JabatoFileFeeder::blockSongs(){
	return block->songsCount();
}

bool
JabatoFileFeeder::blockSongsShuffled(){
  return block->songsShuffled();
}

int
JabatoFileFeeder::blockSlots(){
  return block->slotsCount();
}

char *
JabatoFileFeeder::blockSlotAt(int i){
  return block->getSlotsBlock()->getSoundAt(i);
}

bool
JabatoFileFeeder::blockSlotsShuffled(){
  return block->slotsShuffled();
}

int
JabatoFileFeeder::blockSongsPerSlot(){
  return block->getSongsPerSlot();
}

int
JabatoFileFeeder::blockBeginAtHour(){
  return schedule->getCurrentStrip()->beginAtHour;
}

int
JabatoFileFeeder::blockBeginAtMinute(){
  return schedule->getCurrentStrip()->beginAtMinute;
}

int
JabatoFileFeeder::blockEndAtHour(){
  return schedule->getCurrentStrip()->endAtHour;
}

int
JabatoFileFeeder::blockEndAtMinute(){
  return schedule->getCurrentStrip()->endAtMinute;
}

bool
JabatoFileFeeder::blockUpdated(){
  if(updated) {
    updated = false;
    return true;
  }
  else
    return false;
}

int
JabatoFileFeeder::systemClock(int *hour, int *minute, int *second){
  QDateTime dt = QDateTime::currentDateTime();
  QTime t = dt.time();

  *hour = t.hour();
  *minute = t.minute();
  *second = t.second();

  return dt.date().dayOfWeek();
}

int
JabatoFileFeeder::blockLeftTime(int *h, int *m, int *s) {
  int secs;
  systemClock(h,m,s);
  *s = ((*s) + (*m)*60 + (*h)*3600);
  if((*s)>finalSecond) //end of the day
    *s = (86400-(*s)) + finalSecond;   //86400secs=24hours
  else
    *s = finalSecond - *s;
  secs=*s;
  *m = *s/60;
  *s%=60;
  *h = *m/60;
  *m%=60;
  return secs;
}


bool
JabatoFileFeeder::blockFinished(int leftSeconds) {
  QDateTime dt = QDateTime::currentDateTime();
  QTime t = dt.time();
  QDate d = dt.date();

  currentMinute = t.minute() + leftSeconds/60 + (leftSeconds%60>30?1:0);
  currentHour = t.hour() + currentMinute/60;
  currentMinute %= 60;

  currentDay = d.dayOfWeek();
  if(currentHour==24) {
    currentHour = 0;
    currentDay++;
    if(currentDay==8)
      currentDay = 1;
  }
  return (!schedule->getCurrentStrip()->withinStrip(currentDay, currentHour, currentMinute));
}


void
JabatoFileFeeder::updateSound(JabatoSound *snd, int leftSeconds, int blockIndex) {
  fprintf(stderr, "JFF::updateSound(%d, %d) ",leftSeconds, blockIndex);

  if(blockFinished(leftSeconds)) {
    DEBUG(fprintf(stderr, "(%02d:%02d) Block Finished \n",currentHour, currentMinute));
    int newBlock = schedule->getBlockPosition(currentDay, currentHour, currentMinute);
    if(newBlock==-1) {
      fprintf(stderr,
              "JabatoFileFeeder::updateSound(): CANNOT READ CURRENT BLOCK POSITION!\n");
      m_blockIndex = 0;
      block = schedule->getDefaultBlock();
    }
    else {
      block = schedule->getBlockAt(newBlock);
      if(block->getId()==-1) {
        fprintf(stderr,
                "JabatoFileFeeder::updateSound(): BLOCK %d NOT FOUND!\n", newBlock);
        m_blockIndex = 0;
        block = schedule->getDefaultBlock();
      }
      else
        m_blockIndex = newBlock;
    }
    finalSecond = (schedule->getCurrentStrip()->endAtHour * 3600) +
                  (schedule->getCurrentStrip()->endAtMinute * 60);
    updated = true;
  }
  else {
    if(blockIndex==-1)
      DEBUG(fprintf(stderr, "(%02d:%02d) Current Block %s\n",
                    currentHour, currentMinute,block->getName()));
    else {
      block = schedule->getBlockAt(blockIndex);
      if(block->getId()==-1) {
        fprintf(stderr,
                "JabatoFileFeeder::updateSound(): BLOCK %d NOT FOUND!\n", blockIndex);
        m_blockIndex = 0;
        block = schedule->getDefaultBlock();
      }
      else
        m_blockIndex = blockIndex;
      schedule->getCurrentStrip()->beginAtHour = currentHour;
      schedule->getCurrentStrip()->beginAtMinute = currentMinute;

      updated = true;
    }
  }

  fprintf(stderr, "at %02d:%02d (CB=%s)\n",
          currentHour, currentMinute,block->getName());

  block->updateSound(snd);
}
