#ifndef JABATOSOUND_H
#define JABATOSOUND_H

#include <kprocess.h>
#include <qdatetime.h>
#include <qfileinfo.h>
#include <qframe.h>

#include "oss.h"

#define NOTDEFINED     -1
#define JABATOPROGRAM   1
#define JABATOSONG      2
#define JABATOSLOT      3
#define JABATOTEXT	4

#define FMT_UNDEFINED  0
#define FMT_8BIT       1          /* unsigned */
#define FMT_16BIT      2          /* signed - native byte order */
#define FMT_MULAW      4          /* NOT SUPPORTED (yet) */
#define FMT_ALAW       8          /* NOT SUPPORTED (yet) */

#define FMT_MAX        2
#define RECORDERBUFFERSIZE 65536

struct SOUNDPARAMS {
    int format;
    int channels;
    int rate;
    int blocksize;
    int latency;
};


class Layer {
    public:
    char *mode_name();
    char *layer_name();
    char *version_name();
    char *version_num();
    unsigned int bitrate();
    unsigned int sfreq();
    bool get(char *fname);
    int version;
    int lay;
    int error_protection;
    int bitrate_index;
    int sampling_frequency;
    int padding;
    int extension;
    int mode;
    int mode_ext;
    int copyright;
    int original;
    int emphasis;
    int stereo;
    int seconds;
};


/* ---- file info class ------------------------------------------- */
class JabatoSound {

public:
    JabatoSound();
    virtual ~JabatoSound() {};

    virtual void setFullPath(const char *);

    void    clear();

    void    setSoundType(int);

    bool    hasSound();
    bool    isProgram();
    bool    isSong();
    bool    isSlot();
    bool    isText();

    bool    isOGG();
    bool    isMP3();
    bool    isURL();
    bool    isFESTIVAL();

    
    virtual const char    *getFullPath() const;
    virtual const char    *getRecordPath() const;

    bool    fileExists();
    void    fileDate(int *y, int *m, int *d);

    virtual const char    *getName() const;
    virtual const char    *getDirectory() const;
    virtual const char    *getSubdirectory() const;
    virtual const char    *getPathToSub() const;

    virtual int      getHours() const;
    virtual int      getMinutes() const;
    virtual int      getSeconds() const;
    virtual int      getTimeInSeconds() const;
    void    setTimeInSeconds(int);

    virtual void     updateTime();  

    void     startTimer();
    bool     timerStarted();
    void     stopTimer();
    void     getElapsedTime(int *h, int *m, int *s);
    void     getLeftTime(int *h, int *m, int *s);
    int      getElapsedTimeInSeconds();
    int      getLeftTimeInSeconds();

    void     setRecordingDate();
    void     getRecordedOn(int *y, int *m, int *d);

    void     setToLanDiskCmd(char *);
    void     setToWanDiskCmd(char *);

    bool     execBackupCommands();

private:
    QString mfilepath, mrecordpath, mext, mname, mdir, msubdir, mpath;
    int     mhours, mminutes, mseconds, total, totalsecs;
    int     soundType;
    bool    started;
    
    QString toLanDiskCmd, toWanDiskCmd;

    KShellProcess toLanDiskProcess, toWanDiskProcess;

    QFileInfo   file;
    QFile	qfile;
    QTime       elapsed;
    QDate       recordedOn;
    Layer	mp3info;
};


/* ---- ogg123/mpg123 interface ------------------------------------------ */
class MP3Player : public QObject {
    Q_OBJECT;

public:
    MP3Player(char *dev=NULL);
    virtual ~MP3Player();

    virtual bool     play(JabatoSound *js);
    virtual bool     stop();

private:
    KProcess    *player;
    JabatoSound *jabatoSound;
    char devname[32];
    char oggdevname[32];

public slots:
    void     decoding_finished(KProcess *);

signals:
    void     finished(MP3Player *);

};


class MP3Recorder : public QObject {
    Q_OBJECT;

public:
    MP3Recorder(int rate=22050, int channels=1, int trigger=0, char *dev=NULL);
    virtual ~MP3Recorder();

    void setRecordParams(int rate=22050, int channels=1, int trigger=0);

    void setLevelFrame(QFrame *);
    void setFFTFrame(QFrame *);
    void resizeFrames();

    //virtual bool     start(JabatoSound *js, const char *albumtag=NULL, const char *artisttag=NULL);
    virtual bool     start(JabatoSound *js, const char *albumtag, const char *artisttag);
    virtual void     finish();

private:
    KProcess *encoder;
    JabatoSound *jabatoSound;

//    void renameSoundFile();
    void checkProgramDirectory();

    char buffer[RECORDERBUFFERSIZE];

    int  inBuffer;

    struct SOUNDPARAMS  sndparams;
    int        trigger,wait;
    Soundcard *card;

    int	recording;
    int	encoderReading;
    int	closeStdinPending;
    int	waitingExitNotification;

                           //TODO:
    QWidget  *fftWid;      //QWidget? problems declaring level.h or fft.h in
    QWidget  *levelWid;    //         this file. not time right now for this

    QFrame  *fftFrame;
    QFrame  *levelFrame;

    bool minSignal(void *);

public slots:
    void     new_data(void *data);  	 /* new data from the soundcard */
    void     new_params(struct SOUNDPARAMS *);

    void     bufferjustread(KProcess *); /* encoder read the buffer */

    void     encoding_finished(KProcess *);  /* encoder has finished its work */

    void     check_if_finished();

signals:
    void    started(MP3Recorder *);
    void    hold(MP3Recorder *);
    void    restarted(MP3Recorder *);
    void    finished(MP3Recorder *);
};

#endif
